%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This script enables to generate the dynamic settings files used by Banjo 
% for reconstructing the cell cycle regulatory subnetwork of 
% Saccharomyces cerevisiae
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% define the number nodes of the networks
net_nodes=27; 

% number of tests:  we assume 4 different levels of prior knowledge 
n_tests=4;

for num_test=1:n_tests
    % num_test=1, corresponding to the results obtained by using 0% of
    % edges a priori known
    % num_test=2, corresponding to the results obtained by using 20% of
    % edges a priori known
    % num_test=3, corresponding to the results obtained by using 50% of
    % edges a priori known
    % num_test=4, corresponding to the results obtained by using 70% of
    % edges a priori known        
    % define a string if the prior knowledge is used
    
    flag_pk=true;% a flag false only if no prior knowledge is used
    if num_test==1
        % Don't take into account the prior knowledge
        flag_pk=false;
        str_pk='_pk_0';
    elseif num_test==2
        str_pk='_pk_20';
    elseif num_test==3
        str_pk='_pk_50';
    elseif num_test==4
        str_pk='_pk_70';
    end

    filename_setting=['dyn_sett_sys_Bio_Example',str_pk,'.txt'];

    fid = fopen(filename_setting, 'a+');
    % define the string containing the choosed searcher
    str1='searcherChoice = Greedy';
    format_str='%c';
    fprintf(fid,format_str,str1);
    fprintf(fid,'%c\n','');
    fclose(fid)

    fid = fopen(filename_setting, 'a+');
    % define the string containing the choosed proposer
    str2='proposerChoice = AllLocalMoves';
    format_str='%c';
    fprintf(fid,format_str,str2);
    fprintf(fid,'%c\n','');
    fclose(fid)

    fid = fopen(filename_setting, 'a+');
    % define the string containing the name of the directory where the
    % input files for the current data run are located
    str3=['inputDirectory = data/release2.0/dynamic/input/'...
        'Banjo_data_Bio_Example'];
    format_str='%c';
    fprintf(fid,format_str,str3);
    fprintf(fid,'%c\n','');
    fclose(fid)

    fid = fopen(filename_setting, 'a+');
    % define the string specifying the name of the file that contain
    % the observations data
    str4='observationsFile = data_Bio_Example.txt';
    format_str='%c';
    fprintf(fid,format_str,str4);
    fprintf(fid,'%c\n','');
    fclose(fid)

    fid = fopen(filename_setting, 'a+');
    % define the string containing the name of the directory where the
    % output files for the current data will be placed by Banjo
    str5=['outputDirectory = data/release2.0/dynamic/output/'...
        'Banjo_results_Bio_Example'];
    format_str='%c';
    fprintf(fid,format_str,str5);
    fprintf(fid,'%c\n','');
    fclose(fid)

    fid = fopen(filename_setting, 'a+');
    % define the string containing the name of the (detailed) report
    % file
    str6=['reportFile = results_Bio_Example',str_pk,'.txt'];
    format_str='%c';
    fprintf(fid,format_str,str6);
    fprintf(fid,'%c\n','');
    fclose(fid)

    fid = fopen(filename_setting, 'a+');
    % define the string that containing the number of variables of the
    % underlying problem
    str7=['variableCount = ', num2str(net_nodes)];
    format_str='%c';
    fprintf(fid,format_str,str7);
    fprintf(fid,'%c\n','');
    fclose(fid)

    fid = fopen(filename_setting, 'a+');
    % define the string containing the form of discretization to be
    % applied to all variables
    str8='discretizationPolicy =  i5';
    fprintf(fid,format_str,str8);
    fprintf(fid,'%c\n','');
    fclose(fid)

    fid = fopen(filename_setting, 'a+');
    % define the string that containing the minimum Markov lag
    str9='minMarkovLag = 1';
    fprintf(fid,format_str,str9);
    fprintf(fid,'%c\n','');
    fclose(fid)

    fid = fopen(filename_setting, 'a+');
    % define the string that containing the maximum Markov lag
    str10='maxMarkovLag = 1';
    fprintf(fid,format_str,str10);
    fprintf(fid,'%c\n','');
    fclose(fid)

    fid = fopen(filename_setting, 'a+');
    % define the string that containing the specifying entire sets of
    % parents with Markov lags that need to be included in all networks
    str11='dbnMandatoryIdentityLags =  ';
    fprintf(fid,format_str,str11);
    fprintf(fid,'%c\n','');
    fclose(fid)

    fid = fopen(filename_setting, 'a+');
    % define the string containing the equivalent sample size
    str12='equivalentSampleSize = 1.0';
    fprintf(fid,format_str,str12);
    fprintf(fid,'%c\n','');
    fclose(fid)


    fid = fopen(filename_setting, 'a+');
    % define the string containing the maximum number of parents that a
    % node is allowed to have
    str13_0='maxParentCount = 9';
    fprintf(fid,format_str,str13_0);
    fprintf(fid,'%c\n','');
    fclose(fid)

    if flag_pk
        fid = fopen(filename_setting, 'a+');
        % define the string containing the file that specifies edges
        % that any proposed network must contain.
        str13_1=['mustBePresentEdgesFile = str_Bio_Example'...
            str_pk,'.str'];...
        fprintf(fid,format_str,str13_1);
        fprintf(fid,'%c\n','');
        fclose(fid)
    end
        
    fid = fopen(filename_setting, 'a+');
    % define the string containing the number of highest scoring
    % networks to be tracked during a search.
    str14='nBestNetworks =  10';
    fprintf(fid,format_str,str14);
    fprintf(fid,'%c\n','');
    fclose(fid)

    fid = fopen(filename_setting, 'a+');
    str15='bestNetworksAre = nonIdentical';
    fprintf(fid,format_str,str15);
    fprintf(fid,'%c\n','');
    fclose(fid)

    fid = fopen(filename_setting, 'a+');
    % define the string specifying if Banjo computes the influence
    % scores for the nodes of the top-scoring network
    str16='computeInfluenceScores =	no';
    fprintf(fid,format_str,str16);
    fprintf(fid,'%c\n','');
    fclose(fid)

    fid = fopen(filename_setting, 'a+');
    % define the string specifying the maximum time that a search is
    % scheduled to run
    str17='maxTime = 1.0 h';
    fprintf(fid,format_str,str17);
    fprintf(fid,'%c\n','');
    fclose(fid)

    
    fid = fopen(filename_setting, 'a+');
    % define a string to set to 'no' the precomputeLogGamma,
    % this option uses a lot of memory when the maxParentCount
    % is above 7 parents, as in the case for networks of 20 nodes
    str17_0='precomputeLogGamma = no';
    fprintf(fid,format_str,str17_0);
    fprintf(fid,'%c\n','');
    fclose(fid)


end

