%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This script enables to compute the reconstruced connectivity 
% matrixes and performance indexes for the in-silico networks of ten and 
% twenty nodes, generated by  using the scalefree network generation model 
% proposed by Albert and Barabasi, via the CLR software. For each network,
% different noise levels have been assumed.
% To this aim, the following functions have been used:
% clr.m, matrixPvalue_mod.m  matrixCompare_mod.m and ppv_range_se.m.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% select which networks you want to test
net_nodes=10; % test over the networks of ten nodes
% net_nodes=20; % test over the networks of twenty nodes 

% networks number
n_networks= 20;

% number of tests: for each network, we consider a set of data of 20 time 
% points, assuming there levels of noise (0%, 10%, 30%) 
n_tests=3;

% allocate a memory space to save the output variables 
A_id_cell_clr{n_networks,n_tests}=[]; % cell array containing the 
                                      % reconstructed network, for each 
                                      % test  

% define a flag (do_sim), true if you choose to do the simulation,
% otherwise you can load the files.mat, where the identified variables have 
% been saved
do_sim=false;                                     
                                    
if not(do_sim)
    % load the results of the simulations 
    if net_nodes==10
        load results_net_in_silico_10n\results_clr_10n
    else    
        load results_net_in_silico_20n\results_clr_20n
    end
end
                                       
% define a vector to store the sparsity coefficient for each network
spars_networks=zeros(1,n_networks);

% define start..incr..finish - threshold region to sample
start=1;
incr=1;
finish=100;
precCutoff=5;
n_sample=length(start:incr:finish);
% define matrices to store the results
% for each test we compute a  matrix whose columns contain the values of 
% PPV at fixed sensitivity interval 
thr_se=0.1;  % width of interval in which the sensitivity value varies
vet_val_se=thr_se:thr_se:1; %a vector containing the sensitivity values


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%  COMPUTE THE PERFORMANCE OF CLR  %%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for num_net=1:n_networks 
    % load the data for the current network
    if net_nodes==10
        eval(['load data_net_in_silico_10n\sys_in_silico_10n_net_',...
            num2str(num_net)]);
    else
        eval(['load data_net_in_silico_20n\sys_in_silico_20n_net_',...
            num2str(num_net)]);
    end
    spars_networks(num_net)=spar_coef;
    
    
    for num_test=1:n_tests
        % num_test=1, corresponding to the results obtained by using a data
        %             set of 20 time points, 0% level of noise, 0% level of
        %             prior knowledge             
        % num_test=2, corresponding to the results obtained by filtering
        %             and interpolating a data set of 20 time points, 10%
        %             level of noise, and using 0% level of prior
        %             knowledge 
        % num_test=3, corresponding to the results obtained by filtering
        %             and interpolating a data set of 20 time points, 30%
        %             level of noise, and using 0% level of prior
        %             knowledge 
        
        

        if num_test==1 
            str_level_noise='nonoise';
            Meas_mtx=Meas_mtx_20tp; % each column contains the
            % measurements (20 time points) with 0% noise level for a node 
        elseif num_test==2 
            str_level_noise='noise10';
            Meas_mtx=Meas_mtx_20tp_noise10_smth; % each column contains the
            % measurements (sample_dt*(time_points-1)-1) for a node, 
            % obtained by filtering and interpolating a data set of 20 
            % time points with 10% level of noise
        elseif num_test==3 
            str_level_noise='noise30';    
            Meas_mtx=Meas_mtx_20tp_noise30_smth; % each column contains the
            % measurements (sample_dt*(time_points-1)-1) for a node, 
            % obtained by filtering and interpolating a data set of 20 
            % time points with 30% level of noise
        end
        
        if do_sim
            % Compute the reconstructed network corresponding to the
            % connectivity matrix via CLR
            [Astouffer, MI] = clr(Meas_mtx', 'stouffer', 10, 3);
            A_id_plos = clr(MI, 'plos');
                
            % store the cell array that contains the identified
            % connectivity matrices for each network and test
            A_id_cell_clr{num_net,num_test}=A_id_plos;
        else
            A_id_plos=A_id_cell_clr{num_net,num_test};
            
        end
        
        % evaluation of the performance by calculating the PPV and
        % sensitivity indexes (so we comment some code of the original
        % function matrixPvalue)
        % We do not take into account the elements on diagonal, so we modified
        % the original function matrixCompare
        orig_net=original_network;
        for index=1:n_nodes
            orig_net(index,index)=0;
        end
        [prec, sens] = matrixPvalue_mod(A_id_plos, orig_net, n_nodes,...
            start,incr,finish,precCutoff);
        
        % consider that the network is represented by graph, then for
        % the performance evaulation, the connectivity matrix is
        % undirected; store in two matrix the PPV and Se (prec and sens) 
        % performance indexes for the each network and test for different 
        % values of sparsity of the identified connectivity matrix
        
        eval(['prec_net_plos_',str_level_noise,'(num_net,:)=prec;']);
        eval(['sens_net_plos_',str_level_noise,'(num_net,:)=sens;']);
   
    end
end

% collect in a matrix the performance indexes 
% according to the following criterion: 
% each column contains the values of PPV at fixed sensitivity interval
PPV_se_clr_nonoise= ppv_range_se(vet_val_se,thr_se/2,sens_net_plos_nonoise,...
    prec_net_plos_nonoise);
PPV_se_clr_noise10= ppv_range_se(vet_val_se,thr_se/2,sens_net_plos_noise10,...
    prec_net_plos_noise10);
PPV_se_clr_noise30= ppv_range_se(vet_val_se,thr_se/2,sens_net_plos_noise30,...
    prec_net_plos_noise30);

% Compute the performance of random algorithm without take into account the
% prior knowledge (PK)

% define a mean sparsity coefficient
spar_coef_mean=mean(spars_networks);
PK_f=0;

% Vector of reconstruction probability values
% Each value defines the probability that an element of the randomly 
% reconstructed matrix is zero
x_pr_v=linspace(0,1,100);

% Compute the corresponding vectors of PPV and Sens, for undirected 
% randomly reconstructed networks
TP_und=PK_f*(1-spar_coef.^2)+(1-PK_f)*(1-spar_coef.^2)*(1-x_pr_v.^2);
% TN_und=x_pr_v.^2*spar_coef^2;
FP_und=(1-x_pr_v.^2)*spar_coef^2;
FN_und=(1-PK_f)*x_pr_v.^2*(1-spar_coef^2);

sens_und=TP_und./(TP_und+FN_und);
% spec_und=TN_und./(TN_und+FP_und);
PPV_und=TP_und./(TP_und+FP_und);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%    PLOT RESULTS     %%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

array={'' '0.2' '' '0.4' '' '0.6' '' '0.8' '' '1'};

figure
% plot the performance of CLR assuming 3 different level of noise: 

% 1) 0% level of noise
s1=subplot(3,1,1);
hold on;
grid on
if net_nodes==10
    title('CLR, n=10, 0%Noise, 0%PK','FontSize', 16)
else
    title('CLR, n=20, 0%Noise, 0%PK','FontSize', 16)
end
axis([0 1 0 1]);
boxplot(PPV_se_clr_nonoise,'labels',array)
ylabel('PPV','FontSize', 14)
set(s1,'FontSize',14)
xlabel('Sn','FontSize', 14)

% 2) 10% level of noise
s2=subplot(3,1,2);
hold on;
grid on
if net_nodes==10
    title('CLR, n=10, 10%Noise, 0%PK','FontSize', 16)
else
    title('CLR, n=20, 10%Noise, 0%PK','FontSize', 16)
end
axis([0 1 0 1]);
boxplot(PPV_se_clr_noise10, 'labels',array)
ylabel('PPV','FontSize', 14)
set(s2,'FontSize',14)
xlabel('Sn','FontSize', 14)

% 3) 30% level of noise
s3=subplot(3,1,3);
hold on;
grid on
if net_nodes==10
    title('CLR, n=10, 30%Noise, 0%PK','FontSize', 16)
else
    title('CLR, n=20, 30%Noise, 0%PK','FontSize', 16)
end
axis([0 1 0 1]);
boxplot(PPV_se_clr_noise30,'labels',array)
ylabel('PPV','FontSize', 14)
set(s3,'FontSize',14)
xlabel('Sn','FontSize', 14)

