%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This script enables to compute the reconstruced connectivity matrices,
% the perturbation vectors B and performance indexes for the in-silico 
% networks of ten and twenty nodes, generated by using the scalefree 
% network generation model proposed by Albert and Barabasi, via CORE-NET 
% and CORE-Net_noPA algorithms. 
% For each network, different noise levels and different levels of prior 
% knowledge (PK) about the original network structure have been assumed.
% To this aim, the following  functions have been used:
% main_corenet.m and main_corenet_nopa.m that recall 
% lmi_sign_constr_full_ex.m and lmi_sign_constr_ex.m; estim_err_thr.m; 
% ppv_sens_spec.m.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc


% define the maxinum number of iterations for each identification
max_iter_num=50;

% threshold value for the estimation error 
thr_err=0.1;

% thereshold value for the estimation error used to compare the performance
% with CLR
thr_err_und=0.01;

% select which networks you want to test
net_nodes=10; % test over the networks of ten nodes
% net_nodes=20; % test over the networks of twenty nodes 

% networks number
n_networks= 20;

% number of tests: for each network, we consider a set of data of 20 time 
% points, assuming there levels of noise (0%, 10%, 30%) and two levels of 
% prior knowledge (0% and 30%)
n_tests=6;

% allocate a memory space to save the output variables of the iterative
% procedure
A_id_cell_pa{n_networks,n_tests}=[]; % cell array containing the identified 
                                     % connectivity matrices, obtained by
                                     % using CORE-Net, for each network and 
                                     % test  
                                  
U_id_cell_pa{n_networks,n_tests}=[]; % cell array containing the identified 
                                     % input vectors, obtained by
                                     % using CORE-Net, for each network and 
                                     % test
                                     
est_err_cell_pa{n_networks,n_tests}=[]; % cell array containing the errors 
                                        % of the identified models,obtained
                                        % by using CORE-Net, for each 
                                        % network and test  
                                     
sol_found_pa{n_networks,n_tests}=[]; % cell array containing a flag, true 
                                     % if a solution has been found, 
                                     % obtained by using CORE-Net, for each
                                     % network and test   

A_id_cell_std{n_networks,n_tests}=[]; % cell array containing the 
                                      % identified connectivity matrices,   
                                      % obtained by using CORE-Net_noPA for
                                      % each network and test
                                    
U_id_cell_std{n_networks,n_tests}=[]; % cell array containing the  
                                      % identified input vectors, obtained  
                                      % by using CORE-Net_noPA for
                                      % each network and test
                                    
est_err_cell_std{n_networks,n_tests}=[]; % cell array containing the errors 
                                         % of the identified models,
                                         % obtained by using CORE-Net_noPA 
                                         % for each network and test
                                        
sol_found_std{n_networks,n_tests}=[]; % cell array containing a flag, true
                                      % if a solution has been found,   
                                      % obtained by using CORE-Net_noPA 
                                      % for each network and test

% define a flag (do_sim), true if you choose to do the simulation,
% otherwise you can load the files.mat, where the identified variables have 
% been saved
do_sim=false;                                     
                                    
if not(do_sim)
    % load the results of the simulations 
    if net_nodes==10
        load results_net_in_silico_10n\results_corenet_10n
        load results_net_in_silico_10n\results_corenet_nopa_10n
    else    
        load results_net_in_silico_20n\results_corenet_20n
        load results_net_in_silico_20n\results_corenet_nopa_20n
    end
end

% define a vector to store the sparsity coefficient for each network
spars_networks=zeros(1,n_networks);

% define matrices to store the results 
PPV_pa=zeros(n_networks,n_tests);
Se_pa=zeros(n_networks,n_tests);
PPV_std=zeros(n_networks,n_tests);
Se_std=zeros(n_networks,n_tests);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%    COMPUTE THE PERFORMANCE OF CORE-Net and CORE-Net_noPA    %%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for num_net=1:n_networks 
    % load the data for the current network
    if net_nodes==10
        eval(['load data_net_in_silico_10n\sys_in_silico_10n_net_',...
            num2str(num_net)]);
    else
        eval(['load data_net_in_silico_20n\sys_in_silico_20n_net_',...
            num2str(num_net)]);
    end
    spars_networks(num_net)=spar_coef; 
    
    for num_test=1:n_tests
        % num_test=1, corresponding to the results obtained by using a data
        %             set of 20 time points, 0% level of noise, 0% level of
        %             prior knowledge             
        % num_test=2, corresponding to the results obtained by filtering
        %             and interpolating a data set of 20 time points, 10%
        %             level of noise, and using 0% level of prior
        %             knowledge 
        % num_test=3, corresponding to the results obtained by filtering
        %             and interpolating a data set of 20 time points, 30%
        %             level of noise, and using 0% level of prior
        %             knowledge 
        % num_test=4, corresponding to the results obtained by using a data
        %             set of 20 time points, 0% level of noise, 30% level 
        %             of prior knowledge 
        % num_test=5, corresponding to the results obtained by filtering
        %             and interpolating a data set of 20 time points, 10%
        %             level of noise, and using 30% level of prior
        %             knowledge 
        % num_test=6, corresponding to the results obtained by filtering
        %             and interpolating a data set of 20 time points, 30%
        %             level of noise, and using 30% level of prior
        %             knowledge 
        
        if num_test==1 || num_test==4
            str='nonoise';
            Meas_mtx=Meas_mtx_20tp; % each column contains the
            % measurements (20 time points) with 0% noise level for a node 
        elseif num_test==2 || num_test==5
            str='noise10';
            Meas_mtx=Meas_mtx_20tp_noise10_smth; % each column contains the
            % measurements (sample_dt*(time_points-1)-1) for a node, 
            % obtained by filtering and interpolating a data set of 20 
            % time points with 10% level of noise
        elseif num_test==3 || num_test==6
            str='noise30';    
            Meas_mtx=Meas_mtx_20tp_noise30_smth; % each column contains the
            % measurements (sample_dt*(time_points-1)-1) for a node, 
            % obtained by filtering and interpolating a data set of 20 
            % time points with 30% level of noise
        end
        
        if num_test>3
            sign_constr_list=Constr_mtx_in; % is a matrix with 3 columns
                % containing 30% of the original connections (a-priori
                % knowledge) to be imposed on the matrix  coefficients:
                % first column contains row-index of the constrained
                % coefficient;
                % second column contains column-index of the constrained
                % coefficient;
                % third column contains the type of constraint;
        else
            sign_constr_list=[];% no constraint to be imposed
                                % on the matrix coefficients
        end
        if do_sim
            % Compute the variables to be identified (the connectivity 
            % matrix A and the exogenous perturbation vector B)

            % Inference by using CORE-Net
            [A_id_cell,U_id_cell,est_err_cell,sol_found] = ...
                main_corenet(Meas_mtx,B_vct,sign_constr_list,...
                n_nodes,max_iter_num);
            
            % store the identified variables
            A_id_cell_pa{num_net,num_test}=A_id_cell;
            U_id_cell_pa{num_net,num_test}=U_id_cell;
            est_err_cell_pa{num_net,num_test}=est_err_cell;
            sol_found_pa{num_net,num_test}=sol_found;
        else
            A_id_cell=A_id_cell_pa{num_net,num_test};
            % U_id_cell=U_id_cell_pa{num_net,num_test};
            est_err_cell=est_err_cell_pa{num_net,num_test};
        end
        
        % selection of the network, taking into account the norm of vector
        % of residuals
        idx_sel=estim_err_thr(est_err_cell,A_id_cell,thr_err);
        % evaluation of the performance by calculating the PPV and
        % sensitivity indexes
        Perf_idx= ppv_sens_spec(original_network,A_id_cell{idx_sel});
        
        % for the performance evaluation, consider the case of directed
        % graph representations: the network is represented as a
        % directed graph, the performance indexes (PPV, Sensitivity) do
        % not take into account the estimated weight of the edge, but
        % only its existence;
        % save in two matrix (PPV_pa, Se_pa) the PPV and Sensitivity
        % (Se) performance indexes: the row index corresponds to the
        % network number, the column index corresponds to the kind of
        % test
        PPV_pa(num_net,num_test)=Perf_idx(2,1);
        Se_pa(num_net,num_test)=Perf_idx(2,2);
        
        
        if num_test<=3
            % To allow a comparison with CLR, with no prior knowledge,
            % consider that the network is represented by graph, then for
            % the performance evaulation, the connectivity matrix is
            % undirected; store in two matrix (PPV_pa_und, Se_pa_und)
            % the PPV and Se performance indexes for the each
            % identification until the norm of the vector of residuals has 
            % significantly decreased
            PPV_pa_und=zeros(1,max_iter_num);
            Se_pa_und=zeros(1,max_iter_num);
            idx_sel_und=estim_err_thr(est_err_cell,A_id_cell,thr_err_und);
            for num_id=2:idx_sel_und
                Perf_idx= ppv_sens_spec(original_network,...
                    A_id_cell{num_id});
                PPV_pa_und(num_id)=Perf_idx(1,1);
                Se_pa_und(num_id)=Perf_idx(1,2);
            end
            eval(['PPV_pa_und_',num2str(net_nodes),'_',str,'(num_net,:)=PPV_pa_und;'])
            eval(['Se_pa_und_',num2str(net_nodes),'_',str,'(num_net,:)=Se_pa_und;'])            
        end
        if do_sim
            % Compute the variables to identified (the connectivity matrix
            % A and the exogenous perturbation vector B

            % Inference by using CORE-Net_noPA
            [A_id_cell,U_id_cell,est_err_cell,sol_found] = ...
                main_corenet_nopa(Meas_mtx,B_vct,sign_constr_list,...
                n_nodes,max_iter_num);
            
            % store the identified variables
            A_id_cell_std{num_net,num_test}=A_id_cell;
            U_id_cell_std{num_net,num_test}=U_id_cell;
            est_err_cell_std{num_net,num_test}=est_err_cell;
            sol_found_std{num_net,num_test}=sol_found;
        else
            A_id_cell=A_id_cell_std{num_net,num_test};
            % U_id_cell=U_id_cell_std{num_net,num_test};
            est_err_cell=est_err_cell_std{num_net,num_test};
        end
        
        % selection of the network, taking into account the norm of vector
        % of residuals
        idx_sel=estim_err_thr(est_err_cell,A_id_cell,thr_err);
        % evaluation of the performance by calculating the PPV and
        % sensitivity indexes
        Perf_idx= ppv_sens_spec(original_network,A_id_cell{idx_sel});
        
        % for the performance evaluation, consider the case of directed
        % graph representations: the network is represented as a
        % directed graph, the performance indexes (PPV, Sensitivity) do
        % not take into account the estimated weight of the edge, but
        % only its existence;
        % save in two matrix (PPV_std, Se_std) the PPV and Sensitivity
        % (Se) performance indexes: the row index corresponds to the
        % network number, the column index corresponds to the kind of
        % test
        PPV_std(num_net,num_test)=Perf_idx(2,1);
        Se_std(num_net,num_test)=Perf_idx(2,2);
                
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% Plot the performance for the undirected case %%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

eval(['Se_pa_und_nonoise=Se_pa_und_',num2str(net_nodes),'_nonoise(:,2:end);']);
eval(['Se_pa_und_noise10=Se_pa_und_',num2str(net_nodes),'_noise10(:,2:end);']);
eval(['Se_pa_und_noise30=Se_pa_und_',num2str(net_nodes),'_noise30(:,2:end);']);
eval(['PPV_pa_und_nonoise=PPV_pa_und_',num2str(net_nodes),'_nonoise(:,2:end);']);
eval(['PPV_pa_und_noise10=PPV_pa_und_',num2str(net_nodes),'_noise10(:,2:end);']);
eval(['PPV_pa_und_noise30=PPV_pa_und_',num2str(net_nodes),'_noise30(:,2:end);']);

thr_se=0.1;  % width of interval in which the sensitivity value varies
vet_val_se=thr_se:thr_se:1; %a vector containing the sensitivity values

% collect in a matrix the performance indexes 
% according to the following criterion: 
% each column contains the values of PPV at fixed sensitivity interval
PPV_se_pa_nonoise= ppv_range_se(vet_val_se,thr_se/2,Se_pa_und_nonoise,...
    PPV_pa_und_nonoise);
PPV_se_pa_noise10= ppv_range_se(vet_val_se,thr_se/2,Se_pa_und_noise10,...
    PPV_pa_und_noise10);
PPV_se_pa_noise30= ppv_range_se(vet_val_se,thr_se/2,Se_pa_und_noise30,...
    PPV_pa_und_noise30);

array={'' '0.2' '' '0.4' '' '0.6' '' '0.8' '' '1'};

figure
s1=subplot(3,1,1);
hold on;
grid
if net_nodes==10
    title('CORE-Net, n=10, 0%Noise, 0%PK','FontSize', 16)
else
    title('CORE-Net, n=20, 0%Noise, 0%PK','FontSize', 16)
end
boxplot(PPV_se_pa_nonoise, 'labels',array)
ylabel('PPV','FontSize', 14)
set(s1,'FontSize',14)
xlabel('Sn','FontSize', 14)



s2=subplot(3,1,2);
hold on;
grid
if net_nodes==10
    title('CORE-Net, n=10, 10%Noise, 0%PK','FontSize', 16)
else
    title('CORE-Net, n=20, 10%Noise, 0%PK','FontSize', 16)
end
boxplot(PPV_se_pa_noise10, 'labels',array)
ylabel('PPV','FontSize', 14)
set(s2,'FontSize',14)
xlabel('Sn','FontSize', 14)

s3=subplot(3,1,3);
hold on;
grid
if net_nodes==10
    title('CORE-Net, n=10, 30%Noise, 0%PK','FontSize', 16)
else
    title('CORE-Net, n=20, 30%Noise, 0%PK','FontSize', 16)
end
boxplot(PPV_se_pa_noise30, 'labels',array)
ylabel('PPV','FontSize', 14)
set(s3,'FontSize',14)
xlabel('Sn','FontSize', 14)


% Compute the performance of random algorithm without take into account the
% prior knowledge (PK)

% define a mean sparsity coefficient
spar_coef_mean=mean(spars_networks);
PK_f=0;

% Vector of reconstruction probability values
% Each value defines the probability that an element of the randomly 
% reconstructed matrix is zero
x_pr_v=linspace(0,1,100);

% Compute the corresponding vectors of PPV and Sens, for undirected 
% randomly reconstructed networks
TP_und=PK_f*(1-spar_coef.^2)+(1-PK_f)*(1-spar_coef.^2)*(1-x_pr_v.^2);
% TN_und=x_pr_v.^2*spar_coef^2;
FP_und=(1-x_pr_v.^2)*spar_coef^2;
FN_und=(1-PK_f)*x_pr_v.^2*(1-spar_coef^2);

sens_und=TP_und./(TP_und+FN_und);
% spec_und=TN_und./(TN_und+FP_und);
PPV_und=TP_und./(TP_und+FP_und);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Plot the performance for the directed case  %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Compute the corresponding vectors of PPV and Sens, for directed 
% randomly reconstructed networks

TP_dir=PK_f*(1-spar_coef_mean)+(1-PK_f)*(1-spar_coef_mean)*(1-x_pr_v);
%TN_dir=x_pr_v*spar_coef_mean;
FP_dir=(1-x_pr_v)*spar_coef_mean;
FN_dir=(1-PK_f)*x_pr_v*(1-spar_coef_mean);

sens_dir=TP_dir./(TP_dir+FN_dir);
% spec_dir=TN_dir./(TN_dir+FP_dir);
PPV_dir=TP_dir./(TP_dir+FP_dir);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%    PLOT RESULTS WITHOUT PK  %%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure;
hold on;
grid on
if net_nodes==10
    title('n=10, 0%PK','FontSize', 16)
    axis([0.1 0.3 0.2 0.55]);
elseif net_nodes==20
    title('n=20, 0%PK','FontSize', 16)
    axis([0.02 0.2 0.2 0.55]);
else
    title('0%PK','FontSize', 16)
end
xlabel('Sn','FontSize', 14)
ylabel('PPV','FontSize', 14)
% axis([0.05 0.4 0.2 0.65]);
% plot the performance of the random algorithm
plot(sens_dir,PPV_dir,'r','LineWidth',2);

% plot the performance of the algorithms (CORE-Net and CORE-Net_noPA),  
% averaged of a set of 20 networks, assuming 3 different level of noise: 

% 1) 0% level of noise
plot(mean(Se_pa(:,1)),mean(PPV_pa(:,1)),'MarkerFaceColor',[0 0 1],...
    'MarkerSize',10,'Marker','o','LineWidth',2,'LineStyle','none');
plot(mean(Se_std(:,1)),mean(PPV_std(:,1)),'og','MarkerSize',10,...
    'LineWidth',2);

% 2) 10% level of noise
plot(mean(Se_pa(:,2)),mean(PPV_pa(:,2)),'MarkerFaceColor',[0 0 1],...
    'MarkerSize',10,'Marker','s','LineWidth',2,'LineStyle','none');
plot(mean(Se_std(:,2)),mean(PPV_std(:,2)),'sg','MarkerSize',10,...
    'LineWidth',2);

% 3) 30% level of noise
plot(mean(Se_pa(:,3)),mean(PPV_pa(:,3)),'MarkerFaceColor',[0 0 1],...
    'MarkerSize',10,'Marker','d','LineWidth',2,'LineStyle','none');
plot(mean(Se_std(:,3)),mean(PPV_std(:,3)),'dg','MarkerSize',10,...
    'LineWidth',2);
legend('RANDOM','CORE-Net 0%noise','CORE-Net_{noPA} 0%noise ',...
    'CORE-Net 10%noise','CORE-Net_{noPA} 10%noise ',...
    'CORE-Net 30%noise','CORE-Net_{noPA} 30%noise ','fontsize',16)



% Compute the performance of random algorithm with take into account the
% prior knowledge (PK)
PK_f=0.3;

% Vector of reconstruction probability values
% Each value defines the probability that an element of the randomly 
% reconstructed matrix is zero
x_pr_v=linspace(0,1,100);

% Compute the corresponding vectors of PPV and Sens, for directed 
% randomly reconstructed networks

TP_dir=PK_f*(1-spar_coef_mean)+(1-PK_f)*(1-spar_coef_mean)*(1-x_pr_v);
%TN_dir=x_pr_v*spar_coef_mean;
FP_dir=(1-x_pr_v)*spar_coef_mean;
FN_dir=(1-PK_f)*x_pr_v*(1-spar_coef_mean);

sens_dir=TP_dir./(TP_dir+FN_dir);
% spec_dir=TN_dir./(TN_dir+FP_dir);
PPV_dir=TP_dir./(TP_dir+FP_dir);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%  PLOT RESULTS WITH PK  %%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure;
hold on;
grid on
if net_nodes==10
    title('n=10, 30%PK','FontSize', 16)
    axis([0.3 0.65 0.2 1]);
elseif net_nodes==20
    title('n=20, 30%PK','FontSize', 16)
    axis([0.3 0.5 0.2 1]);
else
    title('30%PK','FontSize', 16)
end
xlabel('Sn','FontSize', 14)
ylabel('PPV','FontSize', 14)
% plot the performance of the random algorithm
plot(sens_dir,PPV_dir,'r','LineWidth',2);

% plot the performance of the algorithms (CORE-Net and CORE-Net_noPA),  
% averaged of a set of 20 networks, assuming 3 different level of noise: 

% 1) 0% level of noise
plot(mean(Se_pa(:,4)),mean(PPV_pa(:,4)),'MarkerFaceColor',[0 0 1],...
    'MarkerSize',10,'Marker','o','LineWidth',2,'LineStyle','none');
plot(mean(Se_std(:,4)),mean(PPV_std(:,4)),'og','MarkerSize',10,...
    'LineWidth',2);

% 2) 10% level of noise
plot(mean(Se_pa(:,5)),mean(PPV_pa(:,5)),'MarkerFaceColor',[0 0 1],...
    'MarkerSize',10,'Marker','s','LineWidth',2,'LineStyle','none');
plot(mean(Se_std(:,5)),mean(PPV_std(:,5)),'sg','MarkerSize',10,...
    'LineWidth',2);

% 3) 30% level of noise
plot(mean(Se_pa(:,6)),mean(PPV_pa(:,6)),'MarkerFaceColor',[0 0 1],...
    'MarkerSize',10,'Marker','d','LineWidth',2,'LineStyle','none');
plot(mean(Se_std(:,6)),mean(PPV_std(:,6)),'dg','MarkerSize',10,...
    'LineWidth',2);
legend('RANDOM','CORE-Net 0%noise','CORE-Net_{noPA} 0%noise ',...
    'CORE-Net 10%noise','CORE-Net_{noPA} 10%noise ',...
    'CORE-Net 30%noise','CORE-Net_{noPA} 30%noise ','fontsize',16)



eval(['spar_coef_mean_',num2str(net_nodes),'=spar_coef_mean;'])
eval(['PPV_pa_',num2str(net_nodes),'=PPV_pa;'])
eval(['PPV_std_',num2str(net_nodes),'=PPV_std;'])
eval(['Se_pa_',num2str(net_nodes),'=Se_pa;'])
eval(['Se_std_',num2str(net_nodes),'=Se_std;'])


