function [A_id_cell A_id_signed]=banjo_results(filename,n_nodes,...
    n_best_net,signed)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% this function allows to store in a cell array the connectivity matrices, 
% corresponding to the highest scoring networks obtained by the software  
% BANJO.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INPUT VARIABLES
%
% filename:     string containing the path of the report file (.txt)  
%               returned by Banjo 
%
% n_nodes:      number of nodes of the network
%
% n_best_net:   number of the highest scoring networks
%
%
% signed:       flag, true if it is of interest to infer also the type of 
%               influence between the nodes, instead, the reconstructed
%               network should be described as a signed digraph
%
% OUTPUT VARIABLES
%
% A_id_cell:    Cell array that contains the best identified connectivity  
%               matrices, we do not consider the weight of the edge,
%               but only its existence, then the network are represented as 
%               directed graphes
%
% A_id_signed:  The signed-connectivity matrix of the best network returned
%               by Banjo
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% Read the txt file
Report=textread(filename,'%s');
% find the final report
idx_final=find(strcmp(Report,'Final'));
if length(idx_final)>1
    idx_final=idx_final(1);
end
str_nodes=num2str(n_nodes);


% allocate a memory space to store the best networks
A_id_cell{n_best_net}=[];

% find the indexes corresponding to the best networks
num_net_find=0;
idx_num_nets=find(strcmp(Report(idx_final:end),str_nodes));
if length(idx_num_nets)>=n_best_net
    idx_nets=idx_num_nets(1:n_best_net);
else
    idx_nets=idx_num_nets(1:end);
    n_best_net=length(idx_num_nets);
end

% check if you find the best networks
for idx=1:length(idx_nets)
    if strcmp(Report(idx_final+idx_nets(idx)),num2str(0));
        num_net_find=1+num_net_find;
    end
end


if num_net_find~=n_best_net
    A_id_cell=[];
    A_id_signed=[];
    return
end

% store in a array the connectivity matrix of the best networks
for idx1=1:n_best_net
    var_count=2;
    Banjo_net=zeros(n_nodes);
    for row=1:n_nodes
        n_link_row=str2double(Report{idx_final+idx_nets(idx1)+var_count});
        link_from_nodes= zeros(n_link_row,1);
        for n_link=1:n_link_row
            link_from_nodes(n_link)=str2double...
                (Report{idx_final+idx_nets(idx1)+var_count+n_link})+1;
        end
        Banjo_net(row,link_from_nodes)=1;
        var_count=var_count+3+n_link_row;
    end
    A_id_cell{idx1}=Banjo_net;
    clear Banjo_net
end

if signed
    % determine the signed-connectivity matrix of the best network
    idx_score=find(strcmp(Report,'Post-processing'));
    var_count_0=7;
    var_count=var_count_0;
    A_id_signed=zeros(n_nodes,n_nodes);
    while (var_count+idx_score)<length(Report)
        idx_col=str2double(Report{idx_score+var_count}(2))+1;
        idx_row=str2double(Report{idx_score+var_count+2}(2))+1;
        link_score=str2double(Report{idx_score+var_count+3});
        if link_score==0
            A_id_signed(idx_row, idx_col)=NaN;
        else
            A_id_signed(idx_row, idx_col)=link_score;
        end
        var_count=var_count+var_count_0;
    end
else
    A_id_signed=[];
end