function idx_sel=estim_err_thr(est_err_cell,A_id_cell,thr_err)

% the function compares the norm of the vector of residuals, generated by
% the identified model, with those one obtained in the two subsequently 
% iterations, if the norm has not significantly decreased, then function
% returns the index corresponding at that iteration, otherwise the
% procedure is iterated.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INPUT VARIABLES
%
% est_err_cell: Cell array that contains the norm of vector of residuals    
%               generated by the identified model at each step 
%
% A_id_cell:    Cell array that contains the identified connectivity  
%               matrices at each step 
%
% thr_err:      Threshold value for the estimation error 
%               
% OUTPUT VARIABLES
%
% idx_sel:      Number of identifications, after those the norm of residuals  
%               has not significantly decreased
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% determine the number of nodes
n_nodes=size(A_id_cell{1},1);

% define a vector that contains the norm of vector of residuals    
% generated by the identified model at each step 
err_dy_net=zeros(1,size(est_err_cell,2));

% define a relative error
err_dy_net_perc=zeros(1,size(est_err_cell,2));

% compute the number of elements (coefficients of the connectivity matrix)
% not equal to zero
nnz_coef=zeros(1,size(est_err_cell,2));
for idx2=1:size(est_err_cell,2)
    err_dy_net(idx2)=est_err_cell{1,idx2}(1);
    % don't consider the first identification
    if idx2>1
        nnz_coef(idx2)=nnz(A_id_cell{idx2})-n_nodes;
    end
end

% To calculate the index corresponding to the selected identification,
% consider the identified networks whose number of elements different to
% zero is major than the number of nodes or the number of nodes plus the
% a priori known edges 
num_id=find(nnz_coef>0);
for idx2=2:size(est_err_cell,2)
    err_dy_net_perc(idx2)=(err_dy_net(idx2-1)-err_dy_net(idx2))/err_dy_net(1);
end

% flag, true if an identification has been selected
idx_found=false;
if not(isempty(num_id))
    % At least one identified connectivity matrix has coefficients not 
    % equal to zero that are not diagonal elements. Indeed the 
    % initialition network for our algorithm has only self loops on
    % every vertex, that means the connectivity matrix is a diagonal 
    % matirx  
    idx2=num_id(1);
    if isfinite(idx2)
        % find the identification
        while idx2<=size(est_err_cell,2)-2&& not(idx_found)
            if  abs(err_dy_net_perc(idx2+1))<thr_err...
                    && abs(err_dy_net_perc(idx2+2))<thr_err
                % for the two subsequently identifications the relative error 
                % has not significantly decreased
                
                % store the identification
                idx_sel=idx2;
                idx_found=true;
            end
            idx2=idx2+1;
        end
    end
    
    if not(idx_found)
        % if the index has not been found, return the index that the norm 
        % of residuals vectors is minimum   
        err_dy_net_sel=zeros(1,size(est_err_cell,2));
        for idx1=1:size(est_err_cell,2)
            if idx1>=num_id(1)
                % considere only the identified networks whose number of 
                % elements different to zero is major than the number of nodes
                err_dy_net_sel(idx1)=err_dy_net(idx1);
            else
                err_dy_net_sel(idx1)=NaN;
            end
        end
        [val_sel idx_sel]=min(err_dy_net_sel);
    end
else
    % the index has not been found
    idx_sel=NaN;
end
