%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This script enables to generate the input file (the data and the 
% constraints to be imposed) that can be used by the software BANJO for 
% reconstructing the cell cycle regulatory subnetwork of 
% Saccharomyces cerevisiae.  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc
clear all


% load the data 
load Bio_Example_data_results\expression_data_subnetwork

% define the number of nodes of the subnetwork
n_nodes=size(Meas_mtx_dec,2);


% Apply a smoothing technique in order to filter the measurement noise
% and increase by interpolation the number of observations
Meas_mtx_smth=smth_int_data(Meas_mtx_dec,3,0);

% define the file name
filename_data=['C:\Users\Francesco\Documents\MATLAB\matlab_source_files'...
'\Banjo_data_Bio_Example\data_Bio_Example.txt'];

% writes data matrices into files (.txt)
dlmwrite(filename_data, Meas_mtx_smth, 'delimiter', '\t','precision', 6)


% number of tests: we assuming different levels of prior
% knowledge (0%, 20%, 50%, 70% PK)
n_tests=4;

% load a cell array containing the Constraints Matrices
load Bio_Example_data_results\Constr_mtx


for num_test=2:n_tests
    
    % Define the matrix containing the edges to be imposed
    Constr_mtx_in=Constr_mtx{num_test};
    
    if num_test==2
        str_pk='_pk_20';
    elseif num_test==3
        str_pk='_pk_50';
    elseif num_test==4
        str_pk='_pk_70';
    end
    
    % generate the file.str containing the specifies edges to be imposed
    filename_prior=['C:\Users\Francesco\Documents\MATLAB'...
        '\matlab_source_files\Banjo_data_Bio_Example\',...
       'str_Bio_Example'...
            ,str_pk,'.str'];
    
    % define the prior known network 
    prior_orig_net=eye(n_nodes);
    for idx1=1:size(Constr_mtx_in)
        prior_orig_net(Constr_mtx_in(idx1,1),...
            Constr_mtx_in(idx1,2))=Constr_mtx_in(idx1,3);
    end
    
    % create the file.str
    fid = fopen(filename_prior, 'a+');
    fprintf(fid,'%d\n',n_nodes);
    fclose(fid)
    for idx2=1:n_nodes        
        fid = fopen(filename_prior, 'a+');
        % find the in-coming edges for each nodes
        link_from_nodes=find(prior_orig_net(idx2,:)); 
        vct_data=[idx2-1, 0, 0, 1, length(link_from_nodes),...
            link_from_nodes-1];
        str_stc='%d %d: %d %d: %d';
        str_dyn=[];
        for idx3=1:length(link_from_nodes)
            str_dyn_old=str_dyn;
            if idx3==length(link_from_nodes)
                str_dyn=[str_dyn_old,' %d\n'];
            else
                str_dyn=[str_dyn_old,' %d'];
            end
            clear str_dyn_old
        end
        format_str=[str_stc, str_dyn];
        clear str_stc str_dyn
        fprintf(fid,format_str,vct_data);
        fclose(fid)
    end
end
