%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This script enables to generate the input files (the data and the 
% constraints to be imposed) that can be used by the software BANJO for
% reconstructing the randomly generated networks, exhibiting
% ErdosRenyi topology, of ten and twenty nodes.  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc
clear all
% select which networks you want to test
net_nodes=10; % test over the networks of ten nodes
%net_nodes=20; % test over the networks of twenty nodes 

% networks number
n_networks= 20;

for num_net=1:n_networks 
    % load the data for the current network
    if net_nodes==10
        eval(['load data_net_in_silico_rand_10n\sys_in_silico_rand_10n_net_',...
            num2str(num_net)]);
    else
        eval(['load data_net_in_silico_rand_20n\sys_in_silico_rand_20n_net_',...
            num2str(num_net)]);
    end
    
    % Increase by interpolation the number of observations also for the
    % data without noise
    Meas_mtx_20tp_nonoise_smth=smth_int_data(Meas_mtx_20tp,3,0);
    
    % define the file names 
    filename_data_nonoise=['C:\Users\Francesco\Documents\MATLAB'...
        '\matlab_source_files\Banjo_data_net_in_silico_rand_',...
        num2str(net_nodes),'n\sys_rand_',num2str(net_nodes),...
        'n_data_nonoise_net_',num2str(num_net),'.txt'];
    filename_data_noise10=['C:\Users\Francesco\Documents\MATLAB'...
        '\matlab_source_files\Banjo_data_net_in_silico_rand_',...
        num2str(net_nodes),'n\sys_rand_',num2str(net_nodes),...
        'n_data_noise10_net_',num2str(num_net),'.txt'];    
    filename_data_noise30=['C:\Users\Francesco\Documents\MATLAB'...
        '\matlab_source_files\Banjo_data_net_in_silico_rand_',...
        num2str(net_nodes),'n\sys_rand_',num2str(net_nodes),...
        'n_data_noise30_net_',num2str(num_net),'.txt'];
    
    % writes data matrices into files (.txt)
    dlmwrite(filename_data_nonoise, Meas_mtx_20tp_nonoise_smth,...
        'delimiter', '\t','precision', 6)
    dlmwrite(filename_data_noise10, Meas_mtx_20tp_noise10_smth,...
        'delimiter', '\t','precision', 6)
    dlmwrite(filename_data_noise30, Meas_mtx_20tp_noise30_smth,...
        'delimiter', '\t','precision', 6)
    
    % generate the file.str containing the specifies edges to be imposed
    filename_prior=['C:\Users\Francesco\Documents\MATLAB'...
        '\matlab_source_files\Banjo_data_net_in_silico_rand_',...
        num2str(net_nodes),'n\sys_rand_',...
        num2str(net_nodes),'n_prior_net_',num2str(num_net),'.str'];
    
    % define the prior known network 
    prior_orig_net=eye(n_nodes);
    for idx1=1:size(Constr_mtx_in)
        prior_orig_net(Constr_mtx_in(idx1,1),...
            Constr_mtx_in(idx1,2))=Constr_mtx_in(idx1,3);
    end
    
    % create the file.str
    fid = fopen(filename_prior, 'a+');
    fprintf(fid,'%d\n',n_nodes);
    fclose(fid)
    for idx2=1:n_nodes        
        fid = fopen(filename_prior, 'a+');
        % find the in-coming edges for each nodes
        link_from_nodes=find(prior_orig_net(idx2,:)); 
        vct_data=[idx2-1, 0, 0, 1, length(link_from_nodes),...
            link_from_nodes-1];
        str_stc='%d %d: %d %d: %d';
        str_dyn=[];
        for idx3=1:length(link_from_nodes)
            str_dyn_old=str_dyn;
            if idx3==length(link_from_nodes)
                str_dyn=[str_dyn_old,' %d\n'];
            else
                str_dyn=[str_dyn_old,' %d'];
            end
            clear str_dyn_old
        end
        format_str=[str_stc, str_dyn];
        clear str_stc str_dyn
        fprintf(fid,format_str,vct_data);
        fclose(fid)
    end
end
