
function PPV_se= ppv_range_se(vet_val_sens,thr_sens,sens_rec,PPV_rec)

% This function allows to collect in a matrix the performance indexes 
% according to the following criterion: 
% each column contains the values of PPV at fixed sensitivity interval 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% INPUT VARIABLES
% 
% vet_val_sens: a vector containing the sensitivity values
%
% thr_sens:     half width of interval where the sensitivity value
%               varies
%
% sens_rec:     a matrix containing the sensitivity values, the rows and 
%               the columns  show respectively the network number and the 
%               number of identification
%
% PPV_rec:      a matrix containing the ppv values, the rows and 
%               the columns  show respectively the network number and the 
%               number of identification
%
% OUTPUT VARIABLES
%
%
%                
% PPV_se:  a matrix whose columns contain the values of PPV at fixed 
%          sensitivity interval 
%                  



PPV_se=NaN(20*51,length(vet_val_sens));

% define a vector containing the number the 
% elements found for each interval of sensitvity
val_at_range_se=zeros(1,length(vet_val_sens));

for idx=1:length(vet_val_sens)
    val_sens= vet_val_sens(idx);
    
    % find the sensitivity values which belong to 
    % ( vet_val_sens(idx)-thr_sens, vet_val_sens(idx)+thr_sens ];  
    if idx==1
        [idx_row,idx_col]= find((sens_rec <= (val_sens +thr_sens) )&(sens_rec > val_sens-2*thr_sens));
    else
        [idx_row,idx_col]= find((sens_rec <= (val_sens +thr_sens) )&(sens_rec > val_sens-thr_sens));
    end
    
    % find the ppv values for the sensitivity values which
    % belong to that interval
    for idx1=1:length(idx_row)
        %         vct_PPV(idx1)=PPV_rec(idx_row(idx1),idx_col(idx1));
        PPV_se(idx1,idx)=PPV_rec(idx_row(idx1),idx_col(idx1));
    end
    
    % store the number of elements for the fixed interval of sensitivity 
    val_at_range_se(idx)=length(idx_row);
    
end
max_idx=max(val_at_range_se);
PPV_se(max_idx+1:end,:)=[];
